require "defines"

GUI = {
	top_flow = "equipment_swap_top_flow",
	top_button = "equipment_swap_top_button",
	toggle = "equipment_swap_toggle_button",
	side_frame = "equipment_swap_side_frame",
	grid = "equipment_swap_grid",
	grid_pos = "equipment_swap_grid_pos",
	load_from_armor = "equipment_swap_load",
	close = "equipment_swap_close",
	equip = "equipment_swap_equip",
	flow = "equipment_swap_flow",
	button_flow = "equipment_swap_button_flow",
	label = "equipment_swap_label",
}

STYLE = {
	top_button = "equipment-open-button",
	combat = "equipment-combat-button",
	default = "equipment-default-button",
	side_frame = "frame_style",
	flow = "es-flow-style",
	label = "frame_caption_label_style",
	warning = "mod_dependency_invalid_label_style",
	grid = "es-grid-style",
	text_button = "es-text-button-style",
}

function init()
	global.grids = global.grids or {}
	global.combat = {}
end

function player_init(player)
	global.grids[player.index] = global.grids[player.index] or {Combat = {dim = {x=5,y=5}, grid = {}}, Default = {dim = {x=5,y=5}, grid = {}}}
	global.combat[player.index] = false
	if player.force.technologies["armor-making-3"].researched then
		create_top(player)
	end
end

function on_configuration_changed(data)
	if data.mod_changes and data.mod_changes["Equipment-Swap"] then
		init()
		for _,player in pairs(game.players) do
			player_init(player)
		end
	end
end

function create_top(player)
	if not player.gui.top[GUI.top_flow] then
		player.gui.top.add{type = "flow", name = GUI.top_flow, direction = "horizontal"}
	end
	if not player.gui.top[GUI.top_flow][GUI.top_button] then
		player.gui.top[GUI.top_flow].add{type = "button", name = GUI.top_button, style = STYLE.top_button}
	end
	local style = STYLE.default
	if global.combat[player.index] then style = STYLE.combat end
	if not player.gui.top[GUI.top_flow][GUI.toggle] then
		player.gui.top[GUI.top_flow].add{type = "button", name = GUI.toggle, style = style}
	end
end


script.on_init(init)
script.on_configuration_changed(on_configuration_changed)
script.on_event(defines.events.on_player_created, function(event)
	player_init(game.get_player(event.player_index))
end)
script.on_event(defines.events.on_research_finished, function(event)
	if event.research.name == "armor-making-3" then
		for _,player in pairs(event.research.force.players) do
			create_top(player)
		end
	end
end)

function create_side(player)
	if player.gui.left[GUI.side_frame] then player.gui.left[GUI.side_frame].destroy() end
	player.gui.left.add{type = "frame", name = GUI.side_frame, style = STYLE.side_frame, direction = "vertical"}

	local armor = player.get_inventory(defines.inventory.player_armor)[1]

	for grid_name, grid_data in pairs(global.grids[player.index]) do

		local flow = player.gui.left[GUI.side_frame].add{type = "flow", name = GUI.flow .. grid_name, direction = "vertical", style = STYLE.flow}
		flow.add{type = "label", caption = grid_name, name = GUI.label .. grid_name, style = STYLE.label}

		if armor.valid_for_read and armor.has_grid then			

			local grid_table = flow.add{type = "table", name = GUI.grid .. grid_name, colspan = math.max(armor.grid.width,grid_data.dim.x), style = STYLE.grid}

			for y = 1,math.max(armor.grid.height,grid_data.dim.y) do
				for x = 1,math.max(armor.grid.width,grid_data.dim.x) do
				
					local out_of_bounds = false
					if x > armor.grid.width or y > armor.grid.height then out_of_bounds = true end
					
					grid_table.add{type = "checkbox", name = GUI.grid_pos .. "x" .. x .. "y" .. y, style = "es-icon-empty", state = out_of_bounds}
					
				end
			end

			for _,equipment in pairs(grid_data.grid) do
				if equipment.shape then
					for dy = 1,equipment.shape.height do
						for dx = 1,equipment.shape.width do
							grid_table[GUI.grid_pos .. "x" .. equipment.position.x + dx .. "y" .. equipment.position.y + dy].style = "es-icon-" .. equipment.name .. "x" .. dx .. "y" .. dy
						end
					end
				end
			end
		else
			flow.add{type = "label", caption = "Armor has no grid", name = GUI.label .. "no_grid_text", style = STYLE.warning}
		end

		local flow2 = player.gui.left[GUI.side_frame].add{type = "flow", name = GUI.button_flow .. grid_name, direction = "horizontal", style = STYLE.flow}
		flow2.add{type = "button", name = GUI.load_from_armor .. grid_name, caption = "Load from current armor", style = STYLE.text_button}
		flow2.add{type = "button", name = GUI.equip .. grid_name, caption = "Equip armor", style = STYLE.text_button}

	end
	player.gui.left[GUI.side_frame].add{type = "button", name = GUI.close, caption = "Exit", style = STYLE.text_button}
end

function load_armor(player,grid_name)
	local armor = player.get_inventory(defines.inventory.player_armor)[1]
	if armor.valid_for_read and armor.has_grid then	
		local new_grid = {}
		local x_max = 0
		local y_max = 0
		for _,equipment in pairs(armor.grid.equipment) do
			table.insert(new_grid,{
				name = equipment.name, 
				position = {
					x = equipment.position.x,
					y = equipment.position.y
					},
				shape = {
					width = equipment.shape.width,
					height = equipment.shape.height
					}
			})
			x_max = math.max(x_max, equipment.position.x + equipment.shape.width)
			y_max = math.max(y_max, equipment.position.y + equipment.shape.height)
			
		end
		global.grids[player.index][grid_name] = {dim = {x = x_max, y = y_max}, grid = new_grid}
		create_side(player)
	else
		error("No valid equipment grid found.")
	end   
end

function equip_armor(player,grid_name)
	local armor = player.get_inventory(defines.inventory.player_armor)[1]

	if armor.valid_for_read and armor.has_grid then
		for _,new_equipment in pairs(global.grids[player.index][grid_name].grid) do
		
			local already_equiped = false
			for _,old_equipment in pairs(armor.grid.equipment) do
				if old_equipment.position.x == new_equipment.position.x and old_equipment.position.y == new_equipment.position.y then
					if old_equipment.name == new_equipment.name then
						already_equiped = true
					end
					break
				end
			end
			
			if not already_equiped and new_equipment.position.x + new_equipment.shape.width - 1 < armor.grid.width and new_equipment.position.y + new_equipment.shape.height - 1 < armor.grid.height then
				if player.remove_item({name = new_equipment.name}) > 0 then
					if not armor.grid.put{name = new_equipment.name, position = new_equipment.position} then

						for x = new_equipment.position.x, new_equipment.position.x + new_equipment.shape.width - 1 do
							for y = new_equipment.position.y, new_equipment.position.y + new_equipment.shape.height - 1 do
							
								local equipment = armor.grid.take{position = {x = x, y = y}}
								if equipment then
									if not player.insert(equipment) then
										player.surface.spill_item_stack(player.position, equipment)
										player.print("Player's inventory full, equipment dropped.")
									end
								end

							end
						end
						armor.grid.put{name = new_equipment.name, position = new_equipment.position}
					end
				else
					player.print("Missing items for equipment.")
				end
			end
		end
	else
		error("No valid equipment grid found.")
	end
end

function edit_armor(player,element)
	local grid_name = element.parent.name:gsub(GUI.grid,"")
	local x,y = element.name:match("x(%d+)y(%d+)")
	x = tonumber(x)
	y = tonumber(y)
	for i,equipment in ipairs(global.grids[player.index][grid_name].grid) do
		if equipment.position.x < x and equipment.position.x + equipment.shape.width >= x then
			if equipment.position.y < y and equipment.position.y + equipment.shape.width >= y then
				table.remove(global.grids[player.index][grid_name].grid,i)
			end
		end
	end
	create_side(player)
end

script.on_event(defines.events.on_gui_click, function(event)
	local element = event.element
	local player = game.get_player(event.player_index)
	local status, err = pcall(function()
		if element.name == GUI.top_button then
			if player.gui.left[GUI.side_frame] then player.gui.left[GUI.side_frame].destroy() 
			else create_side(player) end
			
		elseif element.name == GUI.toggle then
			global.combat[player.index] = not global.combat[player.index]
			
			if global.combat[player.index] then 
				player.gui.top[GUI.top_flow][GUI.toggle].style = STYLE.combat 
				equip_armor(player,"Combat")
			else
				player.gui.top[GUI.top_flow][GUI.toggle].style = STYLE.default
				equip_armor(player,"Default")
			end
			
		elseif element.name:find(GUI.load_from_armor) then
			load_armor(player,element.name:gsub(GUI.load_from_armor,""))

		elseif element.name:find(GUI.equip) then
			local grid_name = element.name:gsub(GUI.equip,"")
			if grid_name == "Combat" then
				global.combat[player.index] = true
				player.gui.top[GUI.top_flow][GUI.toggle].style = STYLE.combat 
			else
				global.combat[player.index] = false
				player.gui.top[GUI.top_flow][GUI.toggle].style = STYLE.default 
			end				
			equip_armor(player,grid_name)
					
		elseif element.name:find(GUI.grid_pos) then
			edit_armor(player,element)

		elseif element.name == GUI.close then
			player.gui.left[GUI.side_frame].destroy() 
		end
	end)
	if not status then player.print(err) end
end)